/*
 * Infra Red protocols decoder
 * Author: Piotr Wjtowicz
 * 2016-2017
 * pw@elportal.pl
 */

#include "menu.h"

static void menu_F_setting(event_t e);
static void menu_L_setting(event_t e);
static void menu_main(event_t e);
static void menu_change_channel(event_t e);
static void menu_A(event_t e);
static void menu_L(event_t e);
static void menu_F(event_t e);
static void menu_sleep(event_t e);
static void next_channel ();

// -----------------------------------------------------------------------------
void init_menu() {
    main_sm.ptr_state = menu_main;
    main_sm.timer_post_event = 0;
}
// -----------------------------------------------------------------------------
inline void handle_menu() {
    if(main_sm.e != EVENT_NONE && main_sm.e != SLEEP_EVENT) {
        main_sm.sleep_timer = SLEEP_EVENT_START;
    }
    main_sm.ptr_state(main_sm.e);
    main_sm.e = EVENT_NONE;
}
// -----------------------------------------------------------------------------
static int cmp_frame_ret(IR_frame_t* e1, IR_frame_t* e2) {
    if(e1->pt == e2->pt &&
       e1->addr == e2->addr &&
       e1->cmd == e2->cmd) {
        return 1;
    }
    return 0; 
}
// -----------------------------------------------------------------------------
static void copy_frame_ret(IR_frame_t* dst, IR_frame_t* src) {
    dst->addr = src->addr;
    dst->cmd = src->cmd;
    dst->pt = src->pt;    
}
// -----------------------------------------------------------------------------
static void next_channel () {
    main_sm.current_channel++;
    if (main_sm.current_channel >= NO_CHANNELS)
        main_sm.current_channel = 0;
    main_sm.mute_timer = MUTE_TIMER_MAX;
}
// -------------- LEVEL_0 -----------------------------
static void menu_sleep(event_t e) {
    ds.dispOn = 0;
    switch(e) {
        case IR_RCVD:
            ds.dispOn = 1;
            setNextState(menu_main);
            break;
        case KEY_PRESSED:
            ds.dispOn = 1;
            setNextState(menu_main);
            sw_sm.eat_pulled_event = 1;
            break;
    }
}
// -------------- LEVEL_0 -----------------------------
static void menu_main(event_t e) {
    ds.blink_disp = 0;
    ds.disp_val = main_sm.current_channel;
    switch(e) {
        case IR_RCVD:
            if(cmp_frame_ret(&ee_record.ir_code_all, &frame_ret)) {
                next_channel();
            } else {
                uint8_t i = 0;
                do {
                    next_channel();
                } while(++i < NO_CHANNELS 
                        && !cmp_frame_ret(&ee_record.ir_code_each[main_sm.current_channel], &frame_ret));
                
            }
            initsm();
            main_sm.timer_post_event = DISP_FLASH_CHANGE;
            ds.dotFlash = 1;
            break;
        case KEY_PULLED:
            next_channel();
            main_sm.timer_post_event = DISP_FLASH_CHANGE;
            break;
        case KEY_P1:
            setNextState(menu_change_channel);
            sw_sm.eat_pulled_event = 1;
            break;
        case POST_EVENT:
            if(ee_record.current_channel != main_sm.current_channel) {
                ee_record.current_channel = main_sm.current_channel;
                ds.dispFlash = 1;
            }
            break;
        case SLEEP_EVENT:
            setNextState(menu_sleep);
            break;
    }
}

// -------------- LEVEL_1 -----------------------------
//modify of given channel: set IR code, clear code, 
static void menu_change_channel(event_t e) {
    ds.blink_disp = 1;
    ds.disp_val = main_sm.current_channel;
    switch(e) {
        case IR_RCVD:
            copy_frame_ret(&ee_record.ir_code_each[main_sm.current_channel], 
                    &frame_ret);
            setNextState(menu_main);
            initsm();
            break;
        case KEY_PULLED:
            setNextState(menu_A);
            break;
        case KEY_P2:
            ee_record.ir_code_each[main_sm.current_channel].pt = NONE;
            sw_sm.eat_pulled_event = 1;
            setNextState(menu_main);
            break;
        case KEY_OFF2:
            setNextState(menu_main);
            break;
    }    
}
// -----------------------------------------------------------------------------
//modify common IR code for all channels
static void menu_A(event_t e) {
    ds.disp_val = 10;
    switch(e) {
        case IR_RCVD:
            copy_frame_ret(&ee_record.ir_code_all, &frame_ret);
            setNextState(menu_main);
            break;
        case KEY_PULLED:
            setNextState(menu_L);
            break;
        case KEY_P2:
            ee_record.ir_code_all.pt = NONE;
            sw_sm.eat_pulled_event = 1;
            setNextState(menu_main);
            break;
        case KEY_OFF2:
            setNextState(menu_main);
            break;
    }    
}
// -----------------------------------------------------------------------------
//modify light intense of display
static void menu_L(event_t e) {
    ds.disp_val = 12;
    switch(e) {
        case KEY_PULLED:
            setNextState(menu_F);
            break;
        case KEY_P1:
            setNextState(menu_L_setting);
            sw_sm.eat_pulled_event = 1;
            break;
        case KEY_OFF2:
            setNextState(menu_main);
            break;
    }    
   
}
// -----------------------------------------------------------------------------
static void nextLightLevel() {    
    ds.light_level_idx++;
    if(ds.light_level_idx >= LIGHT_LEVELS_N) {
        ds.light_level_idx = 0;
    }
}

static void return_l(void (*nf)(event_t e)) {
    setNextState(nf);
    ds.dotOn = 0;
    if(ds.light_level_idx != ee_record.light_level_idx) {
        ee_record.light_level_idx = ds.light_level_idx;
    }
}

static void menu_L_setting(event_t e) {
    ds.disp_val = LIGHT_LEVELS_N - ds.light_level_idx - 1;
    ds.dotOn = 1;
    switch(e) {
        case KEY_PULLED:
            nextLightLevel();
            break;
        case KEY_P1:
            return_l(menu_L);
            sw_sm.eat_pulled_event = 1;
            break;
        case KEY_OFF2:
            return_l(menu_main);
            break;
    }    
}
// -----------------------------------------------------------------------------
//modify display auto-off some time after user interact
static void menu_F(event_t e) {
    ds.disp_val = 13;
    switch(e) {
        case KEY_PULLED:
            setNextState(menu_change_channel);
            break;
        case KEY_P1:
            setNextState(menu_F_setting);
            sw_sm.eat_pulled_event = 1;
            break;
        case KEY_OFF2:
            setNextState(menu_main);
            break;
    }    
}

static void return_f(void (*fe)(event_t e)) {
    setNextState(fe);
    if(ee_record.auto_sleep_mode != main_sm.sleep_timer_step) {
        ee_record.auto_sleep_mode = main_sm.sleep_timer_step;
    }
    ds.dotOn = 0;
}

static void menu_F_setting(event_t e) {
    ds.disp_val = main_sm.sleep_timer_step;
    ds.dotOn = 1;
    switch(e) {
        case KEY_PULLED:
            main_sm.sleep_timer_step = (main_sm.sleep_timer_step) ? 0 : 1;
            break;
        case KEY_P1:
            return_f(menu_F);
            sw_sm.eat_pulled_event = 1;
            break;
        case KEY_OFF2:
            return_f(menu_main);
            break;
    }       
}
// -----------------------------------------------------------------------------